@@ -1,7 +1,7 @@
 /*
  *  editdlg.cpp  -  dialogue to create or modify an alarm or alarm template
  *  Program:  kalarm
- *  Copyright (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2006 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -135,6 +135,7 @@ inline QString recurText(const KAEvent& event)
 // translations across different modules.
 QString EditAlarmDlg::i18n_ConfirmAck()         { return i18n("Confirm acknowledgment"); }
 QString EditAlarmDlg::i18n_k_ConfirmAck()       { return i18n("Confirm ac&knowledgment"); }
+QString EditAlarmDlg::i18n_SpecialActions()     { return i18n("Special Actions..."); }
 QString EditAlarmDlg::i18n_ShowInKOrganizer()   { return i18n("Show in KOrganizer"); }
 QString EditAlarmDlg::i18n_g_ShowInKOrganizer() { return i18n("Show in KOr&ganizer"); }
 QString EditAlarmDlg::i18n_EnterScript()        { return i18n("Enter a script"); }
@@ -266,7 +267,7 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 	grid->setRowStretch(2, 1);
 	mAlarmTypeStack->addWidget(mDisplayAlarmsFrame, 0);
 	mAlarmTypeStack->addWidget(mCommandFrame, 1);
-	mAlarmTypeStack->addWidget(mEmailFrame, 1);
+	mAlarmTypeStack->addWidget(mEmailFrame, 2);
 
 	// Deferred date/time: visible only for a deferred recurring event.
 	mDeferGroup = new QGroupBox(1, Qt::Vertical, i18n("Deferred Alarm"), mainPage, "deferGroup");
@@ -366,6 +367,7 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 	      i18n("How often the alarm recurs.\nThe times shown are those configured in the Recurrence tab and in the Simple Repetition dialog."));
 	box->setFixedHeight(box->sizeHint().height());
 	layout->addWidget(box);
+	layout->addStretch();
 
 	// Simple repetition button
 	mSimpleRepetition = new RepetitionButton(i18n("Simple Repetition"), true, mainPage);
@@ -375,19 +377,27 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 	QWhatsThis::add(mSimpleRepetition, i18n("Set up a simple, or additional, alarm repetition"));
 	layout->addWidget(mSimpleRepetition);
 
+	// Reminder
+	static const QString reminderText = i18n("Enter how long in advance of the main alarm to display a reminder alarm.");
+	mReminder = new Reminder(i18n("Rem&inder:"),
+	                         i18n("Check to additionally display a reminder in advance of the main alarm time(s)."),
+	                         QString("%1\n\n%2").arg(reminderText).arg(TimeSpinBox::shiftWhatsThis()),
+	                         true, true, mainPage);
+	mReminder->setFixedSize(mReminder->sizeHint());
+	topLayout->addWidget(mReminder, 0, Qt::AlignAuto);
+
+	// Late cancel selector - default = allow late display
+	mLateCancel = new LateCancelSelector(true, mainPage);
+	topLayout->addWidget(mLateCancel, 0, Qt::AlignAuto);
+
 	if (theApp()->korganizerEnabled())
 	{
 		// Show in KOrganizer checkbox
 		mShowInKorganizer = new CheckBox(i18n_ShowInKOrganizer(), mainPage);
 		mShowInKorganizer->setFixedSize(mShowInKorganizer->sizeHint());
 		QWhatsThis::add(mShowInKorganizer, i18n("Check to copy the alarm into KOrganizer's calendar"));
-		layout->addWidget(mShowInKorganizer);
+		topLayout->addWidget(mShowInKorganizer);
 	}
-	layout->addStretch();
-
-	// Late cancel selector - default = allow late display
-	mLateCancel = new LateCancelSelector(true, mainPage);
-	topLayout->addWidget(mLateCancel, 0, Qt::AlignAuto);
 
 	setButtonWhatsThis(Ok, i18n("Schedule the alarm at the specified time."));
 
@@ -397,7 +407,7 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 		mTemplateName->setFocus();
 
 	// Save the initial state of all controls so that we can later tell if they have changed
-	saveState(event);
+	saveState((event && (mTemplate || !event->isTemplate())) ? event : 0);
 
 	// Note the current desktop so that the dialog can be shown on it.
 	// If a main window is visible, the dialog will by KDE default always appear on its
@@ -462,15 +472,6 @@ void EditAlarmDlg::initDisplayAlarms(QWidget* parent)
 	mSoundPicker->setFixedSize(mSoundPicker->sizeHint());
 	frameLayout->addWidget(mSoundPicker, 0, Qt::AlignAuto);
 
-	// Reminder
-	static const QString reminderText = i18n("Enter how long in advance of the main alarm to display a reminder alarm.");
-	mReminder = new Reminder(i18n("Rem&inder:"),
-	                         i18n("Check to additionally display a reminder in advance of the main alarm time(s)."),
-	                         QString("%1\n\n%2").arg(reminderText).arg(TimeSpinBox::shiftWhatsThis()),
-	                         true, true, mDisplayAlarmsFrame);
-	mReminder->setFixedSize(mReminder->sizeHint());
-	frameLayout->addWidget(mReminder, 0, Qt::AlignAuto);
-
 	// Acknowledgement confirmation required - default = no confirmation
 	layout = new QHBoxLayout(frameLayout);
 	mConfirmAck = createConfirmAckCheckbox(mDisplayAlarmsFrame);
@@ -482,7 +483,7 @@ void EditAlarmDlg::initDisplayAlarms(QWidget* parent)
 	if (ShellProcess::authorised())    // don't display if shell commands not allowed (e.g. kiosk mode)
 	{
 		// Special actions button
-		mSpecialActionsButton = new SpecialActionsButton(i18n("Special Actions..."), mDisplayAlarmsFrame);
+		mSpecialActionsButton = new SpecialActionsButton(i18n_SpecialActions(), mDisplayAlarmsFrame);
 		mSpecialActionsButton->setFixedSize(mSpecialActionsButton->sizeHint());
 		layout->addWidget(mSpecialActionsButton);
 	}
@@ -1059,7 +1060,7 @@ void EditAlarmDlg::saveState(const KAEvent* event)
 		mSavedEmailAttach += mEmailAttachList->text(i);
 	mSavedEmailBcc         = mEmailBcc->isChecked();
 	if (mTimeWidget)
-		mSavedDateTime = mTimeWidget->getDateTime(false, false);
+		mSavedDateTime = mTimeWidget->getDateTime(0, false, false);
 	mSavedLateCancel       = mLateCancel->minutes();
 	mSavedAutoClose        = mLateCancel->isAutoClose();
 	if (mShowInKorganizer)
@@ -1093,7 +1094,7 @@ bool EditAlarmDlg::stateChanged() const
 			return true;
 	}
 	else
-		if (mSavedDateTime != mTimeWidget->getDateTime(false, false))
+		if (mSavedDateTime != mTimeWidget->getDateTime(0, false, false))
 			return true;
 	if (mSavedTypeRadio        != mActionGroup->selected()
 	||  mSavedLateCancel       != mLateCancel->minutes()
@@ -1402,7 +1403,7 @@ void EditAlarmDlg::slotOk()
 	else
 	{
 		QWidget* errWidget;
-		mAlarmDateTime = mTimeWidget->getDateTime(!(timedRecurrence || repeated), false, &errWidget);
+		mAlarmDateTime = mTimeWidget->getDateTime(0, !(timedRecurrence || repeated), false, &errWidget);
 		if (errWidget)
 		{
 			// It's more than just an existing deferral being changed, so the time matters
@@ -1421,14 +1422,16 @@ void EditAlarmDlg::slotOk()
 		{
 			QDateTime now = QDateTime::currentDateTime();
 			if (mAlarmDateTime.date() < now.date()
-			||  !mAlarmDateTime.isDateOnly() && mAlarmDateTime.time() < now.time())
+			||  mAlarmDateTime.date() == now.date()
+			    && !mAlarmDateTime.isDateOnly() && mAlarmDateTime.time() < now.time())
 			{
 				// A timed recurrence has an entered start date which
 				// has already expired, so we must adjust it.
 				KAEvent event;
 				getEvent(event);     // this may adjust mAlarmDateTime
-				if ((mAlarmDateTime.date() < now.date()
-				     ||  !mAlarmDateTime.isDateOnly() && mAlarmDateTime.time() < now.time())
+				if ((  mAlarmDateTime.date() < now.date()
+				    || mAlarmDateTime.date() == now.date()
+				       && !mAlarmDateTime.isDateOnly() && mAlarmDateTime.time() < now.time())
 				&&  event.nextOccurrence(now, mAlarmDateTime, KAEvent::ALLOW_FOR_REPETITION) == KAEvent::NO_OCCURRENCE)
 				{
 					KMessageBox::sorry(this, i18n("Recurrence has already expired"));
@@ -1570,7 +1573,7 @@ void EditAlarmDlg::slotEditDeferral()
 		return;
 	bool limit = true;
 	int repeatCount = mSimpleRepetition->count();
-	DateTime start = mTimeWidget->getDateTime(!repeatCount, !mExpiredRecurrence);
+	DateTime start = mTimeWidget->getDateTime(0, !repeatCount, !mExpiredRecurrence);
 	if (!start.isValid())
 	{
 		if (!mExpiredRecurrence)
@@ -1652,7 +1655,7 @@ void EditAlarmDlg::slotShowRecurrenceEdit()
 	if (!mReadOnly  &&  !mTemplate)
 	{
 		QDateTime now = QDateTime::currentDateTime();
-		mAlarmDateTime = mTimeWidget->getDateTime(false, false);
+		mAlarmDateTime = mTimeWidget->getDateTime(0, false, false);
 		bool expired = (mAlarmDateTime.dateTime() < now);
 		if (mRecurSetDefaultEndDate)
 		{
@@ -1681,7 +1684,7 @@ void EditAlarmDlg::slotRecurTypeChange(int repeatType)
 		bool atLogin = (mRecurrenceEdit->repeatType() == RecurrenceEdit::AT_LOGIN);
 		if (atLogin)
 		{
-			mAlarmDateTime = mTimeWidget->getDateTime(false, false);
+			mAlarmDateTime = mTimeWidget->getDateTime(0, false, false);
 			mRecurrenceEdit->setEndDateTime(mAlarmDateTime.dateTime());
 		}
 		mReminder->enableOnceOnly(recurs && !atLogin);
@@ -1829,6 +1832,7 @@ void EditAlarmDlg::slotAlarmTypeChanged(int)
 		mFilePadding->hide();
 		mTextMessageEdit->show();
 		mFontColourButton->show();
+		mReminder->show();
 		mSoundPicker->showSpeak(true);
 		setButtonWhatsThis(Try, i18n("Display the alarm message now"));
 		mAlarmTypeStack->raiseWidget(mDisplayAlarmsFrame);
@@ -1841,6 +1845,7 @@ void EditAlarmDlg::slotAlarmTypeChanged(int)
 		mFileBox->show();
 		mFilePadding->show();
 		mFontColourButton->hide();
+		mReminder->show();
 		mSoundPicker->showSpeak(false);
 		setButtonWhatsThis(Try, i18n("Display the file now"));
 		mAlarmTypeStack->raiseWidget(mDisplayAlarmsFrame);
@@ -1850,6 +1855,7 @@ void EditAlarmDlg::slotAlarmTypeChanged(int)
 	}
 	else if (mCommandRadio->isOn())
 	{
+		mReminder->hide();
 		setButtonWhatsThis(Try, i18n("Execute the specified command now"));
 		mAlarmTypeStack->raiseWidget(mCommandFrame);
 		mCmdCommandEdit->setNoSelect();
@@ -1857,6 +1863,7 @@ void EditAlarmDlg::slotAlarmTypeChanged(int)
 	}
 	else if (mEmailRadio->isOn())
 	{
+		mReminder->hide();
 		setButtonWhatsThis(Try, i18n("Send the email to the specified addressees now"));
 		mAlarmTypeStack->raiseWidget(mEmailFrame);
 		mEmailToEdit->setNoSelect();
