@@ -1,5 +1,5 @@
 /* -*- Mode: C++ -*-
-   $Id: KDGanttViewSubwidgets.h,v 1.3 2004/03/21 13:34:17 rogowski Exp $
+   $Id: KDGanttViewSubwidgets.h 516548 2006-03-07 14:58:04Z danders $
    KDGantt - a multi-platform charting engine
 */
 
@@ -54,6 +54,7 @@
 #include <qbrush.h>
 #include <qvbox.h>
 #include <qdockwindow.h>
+#include <qtimer.h>
 
 #include "KDGanttView.h"
 #include "KDGanttViewTaskLink.h"
@@ -238,7 +239,6 @@ private:
     int endMouseDown;
     bool autoComputeTimeLine;
     QPixmap paintPix;
-
 };
 
 /* KDTimeTableWidget */
@@ -258,6 +258,7 @@ public:
     void setShowTaskLinks( bool show );
     bool showTaskLinks();
     QPtrList<KDGanttViewTaskLink>taskLinks();
+    void clearTaskLinks();
     void updateMyContent();
     void removeItemFromTasklinks( KDGanttViewItem * );
     void setHorBackgroundLines( int count, QBrush brush );
@@ -266,6 +267,8 @@ public:
     void setNoInformationBrush( const QBrush& brush );
     QBrush noInformationBrush() const;
 
+    int getCoordX( QDateTime dt );
+    
 signals:
    void   heightComputed( int );
 
@@ -350,7 +353,7 @@ class KDListView : public QListView
 public:
    KDListView (QWidget* parent,KDGanttView* gv );
    KDGanttView* myGanttView;
-   void drawToPainter( QPainter *p );
+   void drawToPainter( QPainter *p, bool drawHeader=false );
    void setCalendarMode( bool mode );
   bool calendarMode() { return _calendarMode; };
   QString getWhatsThisText(QPoint p);
@@ -362,6 +365,19 @@ public:
   QDragObject * dragObject ();
   void startDrag ();
   void paintemptyarea ( QPainter * p, const QRect & rect ){ QListView::paintEmptyArea( p, rect );};
+
+public:
+    class DrawableItem {
+    public:
+        DrawableItem(int level, int ypos, QListViewItem *item ) { y = ypos; l = level; i = item; };
+        int y;
+        int l;
+        QListViewItem * i;
+    };
+protected:
+    void drawAllContents(QPainter * p, int cx, int cy, int cw, int ch);
+    int buildDrawables(QPtrList<KDListView::DrawableItem> &lst, int level, int ypos, QListViewItem *item, int ymin, int ymax) const;
+
 private slots:
   void dragItem( QListViewItem * );
  private:
@@ -468,6 +484,14 @@ protected:
     QPopupMenu* onItem;
     bool _showItemAddPopupMenu;
     int myMyContentsHeight;
+    KDGanttViewItem *fromItem;
+    bool linkItemsEnabled;
+    QCanvasLine *linkLine;
+    int fromArea;
+    bool autoScrollEnabled;
+    int getItemArea(KDGanttViewItem *item, int x);
+    int getLinkType(int from, int to);
+
 signals:
   void heightResized( int );
   void widthResized( int );
@@ -475,14 +499,20 @@ public slots:
   void set_Mouse_Tracking(bool on);
   void moveMyContent( int, int );
   void setMyContentsHeight( int );
+  void updateHorScrollBar();
 private slots:
   void cutItem();
   void pasteItem( int );
   void newRootItem( int );
   void newChildItem( int );
-
+  void slotScrollTimer();
+  void myUpdateScrollBars();
+  
 private:
   KDCanvasToolTip* myToolTip;
+  QTimer *myScrollTimer;
+  QPoint mousePos;
+  QTimer scrollBarTimer;
 };
 
 class KDTimeHeaderToolTip :public QToolTip
