@@ -144,7 +144,7 @@ KMFolderCachedImap::KMFolderCachedImap( KMFolder* folder, const char* aName )
     mSubfolderState( imapNoInformation ),
     mIncidencesFor( IncForAdmins ),
     mIsSelected( false ),
-    mCheckFlags( true ), mAccount( NULL ), uidMapDirty( true ),
+    mCheckFlags( true ), mReadOnly( false ), mAccount( NULL ), uidMapDirty( true ),
     uidWriteTimer( -1 ), mLastUid( 0 ), mTentativeHighestUid( 0 ),
     mUserRights( 0 ), mSilentUpload( false ),
     mFolderRemoved( false ),
@@ -539,8 +539,11 @@ void KMFolderCachedImap::serverSync( bool recurse )
   mRecurse = recurse;
   assert( account() );
 
-  mAccount->mailCheckProgressItem()->reset();
-  mAccount->mailCheckProgressItem()->setTotalItems( 100 );
+  ProgressItem *progressItem = mAccount->mailCheckProgressItem();
+  if ( progressItem ) {
+    progressItem->reset();
+    progressItem->setTotalItems( 100 );
+  }
   mProgress = 0;
 
 #if 0
@@ -1115,12 +1118,12 @@ void KMFolderCachedImap::uploadNewMessages()
       job->start();
       return;
     } else {
-      const QString msg ( i18n( "<p>There are new messages in this folder, which "
+      const QString msg ( i18n( "<p>There are new messages in this folder (%1), which "
             "have not been uploaded to the server yet, but you do not seem to "
             "have sufficient access rights on the folder now to upload them. "
             "Please contact your administrator to allow upload of new messages "
             "to you, or move them out of this folder.</p> "
-            "<p>Do you want to move those messages to another folder now?</p>") );
+            "<p>Do you want to move these messages to another folder now?</p>").arg( folder()->prettyURL() ) );
       if ( KMessageBox::warningYesNo( 0, msg, QString::null, i18n("Move"), i18n("Do Not Move") ) == KMessageBox::Yes ) {
         KMail::KMFolderSelDlg dlg( kmkernel->getKMMainWidget(),
             i18n("Move Messages to Folder"), true );
@@ -1205,6 +1208,10 @@ void KMFolderCachedImap::uploadFlags()
 
 void KMFolderCachedImap::slotImapStatusChanged(KMFolder* folder, const QString&, bool cont)
 {
+  if ( mSyncState == SYNC_STATE_INITIAL ){
+      kdDebug(5006) << "IMAP status changed but reset " << endl;
+      return; // we were reset
+  }
   if ( folder->storage() == this ) {
     --mStatusFlagsJobs;
     if ( mStatusFlagsJobs == 0 || !cont ) // done or aborting
@@ -1746,11 +1753,14 @@ void KMFolderCachedImap::listDirectory2()
         f = static_cast<KMFolderCachedImap*>(newFolder->storage());
       }
     }
-    f->setAccount( mAccount );
-    f->setImapPath( "/INBOX/" );
-    f->folder()->setLabel( i18n("inbox") );
+    if ( f ) {
+      f->setAccount( mAccount );
+      f->setImapPath( "/INBOX/" );
+      f->folder()->setLabel( i18n("inbox") );
+    }
     if (!node) {
-      f->close();
+      if ( f )
+        f->close();
       kmkernel->dimapFolderMgr()->contentsChanged();
     }
     // so we have an INBOX
@@ -2004,6 +2014,7 @@ KMFolderCachedImap::slotACLChanged( const QString& userId, int permissions )
 // called by KMAcctCachedImap::killAllJobs
 void KMFolderCachedImap::resetSyncState()
 {
+  if ( mSyncState == SYNC_STATE_INITIAL ) return;
   mSubfoldersForSync.clear();
   mSyncState = SYNC_STATE_INITIAL;
   close();
