@@ -1,8 +1,6 @@
 /*
-    kngrouppropdlg.cpp
-
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2005 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -11,7 +9,7 @@
     (at your option) any later version.
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software Foundation,
-    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, US
+    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, US
 */
 
 #include <qgroupbox.h>
@@ -24,6 +22,7 @@
 
 #include "knglobals.h"
 #include "knconfigmanager.h"
+#include "knconfigwidgets.h"
 #include "utilities.h"
 #include "kngroup.h"
 #include "kngrouppropdlg.h"
