@@ -1,7 +1,7 @@
 /*******************************************************************
  KNotes -- Notes for the KDE project
 
- Copyright (c) 1997-2005, The KNotes Developers
+ Copyright (c) 1997-2006, The KNotes Developers
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
@@ -110,6 +110,7 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
         this, SLOT(slotRename()), actionCollection(), "rename_note" );
     m_readOnly = new KToggleAction( i18n("Lock"), "lock" , 0,
         this, SLOT(slotUpdateReadOnly()), actionCollection(), "lock_note" );
+    m_readOnly->setCheckedState( KGuiItem( i18n("Unlock"), "unlock" ) );
     new KAction( i18n("Hide"), "fileclose" , Key_Escape,
         this, SLOT(slotClose()), actionCollection(), "hide_note" );
     new KAction( i18n("Delete"), "knotes_delete", 0,
@@ -171,6 +172,7 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     m_tool = static_cast<KToolBar*>(factory.container( "note_tool", this ));
     m_tool->setIconSize( 10 );
     m_tool->setFixedHeight( 16 );
+    m_tool->setIconText( KToolBar::IconOnly );
 
     // if there was just a way of making KComboBox adhere the toolbar height...
     QObjectList *list = m_tool->queryList( "KComboBox" );
@@ -283,16 +285,6 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     height = m_config->height();
     resize( width, height );
 
-    if ( m_config->keepAbove() )
-        m_keepAbove->setChecked( true );
-    else if ( m_config->keepBelow() )
-        m_keepBelow->setChecked( true );
-    else
-    {
-        m_keepAbove->setChecked( false );
-        m_keepBelow->setChecked( false );
-    }
-
     // let KWin do the placement if the position is illegal--at least 10 pixels
     // of a note need to be visible
     const QPoint& position = m_config->position();
@@ -351,6 +343,17 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     m_readOnly->setChecked( m_config->readOnly() );
     slotUpdateReadOnly();
 
+    if ( m_config->keepAbove() )
+        m_keepAbove->setChecked( true );
+    else if ( m_config->keepBelow() )
+        m_keepBelow->setChecked( true );
+    else
+    {
+        m_keepAbove->setChecked( false );
+        m_keepBelow->setChecked( false );
+    }
+    slotUpdateKeepAboveBelow();
+
     // HACK: update the icon color - again after showing the note, to make kicker aware of the new colors
     KIconEffect effect;
     QPixmap icon = effect.apply( kapp->icon(), KIconEffect::Colorize, 1, m_config->bgColor(), false );
@@ -370,9 +373,10 @@ void KNote::slotKill( bool force )
 {
     if ( !force &&
          KMessageBox::warningContinueCancel( this,
-            i18n("<qt>Do you really want to delete note <b>%1</b>?</qt>")
-                .arg( m_label->text() ),
-            i18n("Confirm Delete"), KGuiItem( i18n("&Delete"), "editdelete") )
+             i18n("<qt>Do you really want to delete note <b>%1</b>?</qt>").arg( m_label->text() ),
+             i18n("Confirm Delete"), KGuiItem( i18n("&Delete"), "editdelete" ),
+             "ConfirmDeleteNote"
+         )
          != KMessageBox::Continue )
     {
         return;
@@ -470,6 +474,80 @@ void KNote::setText( const QString& text )
     saveData();
 }
 
+QColor KNote::fgColor() const
+{
+    return m_config->fgColor();
+}
+
+QColor KNote::bgColor() const
+{
+    return m_config->bgColor();
+}
+
+void KNote::setColor( const QColor& fg, const QColor& bg )
+{
+    m_journal->setCustomProperty( "KNotes", "FgColor", fg.name() );
+    m_journal->setCustomProperty( "KNotes", "BgColor", bg.name() );
+    m_config->setFgColor( fg );
+    m_config->setBgColor( bg );
+
+    m_journal->updated();  // because setCustomProperty() doesn't call it!!
+    emit sigDataChanged();
+    m_config->writeConfig();
+
+    QPalette newpalette = palette();
+    newpalette.setColor( QColorGroup::Background, bg );
+    newpalette.setColor( QColorGroup::Foreground, fg );
+    newpalette.setColor( QColorGroup::Base,       bg ); // text background
+    newpalette.setColor( QColorGroup::Text,       fg ); // text color
+    newpalette.setColor( QColorGroup::Button,     bg );
+    newpalette.setColor( QColorGroup::ButtonText, fg );
+
+//    newpalette.setColor( QColorGroup::Highlight,  bg );
+//    newpalette.setColor( QColorGroup::HighlightedText, fg );
+
+    // the shadow
+    newpalette.setColor( QColorGroup::Midlight, bg.light(150) );
+    newpalette.setColor( QColorGroup::Shadow, bg.dark(116) );
+    newpalette.setColor( QColorGroup::Light, bg.light(180) );
+    if ( s_ppOffset )
+        newpalette.setColor( QColorGroup::Dark, bg.dark(200) );
+    else
+        newpalette.setColor( QColorGroup::Dark, bg.dark(108) );
+    setPalette( newpalette );
+
+    // set the text color
+    m_editor->setTextColor( fg );
+
+    // set the background color or gradient
+    updateBackground();
+
+    // set darker value for the hide button...
+    QPalette darker = palette();
+    darker.setColor( QColorGroup::Button, bg.dark(116) );
+    m_button->setPalette( darker );
+
+    // update the icon color
+    KIconEffect effect;
+    QPixmap icon = effect.apply( kapp->icon(), KIconEffect::Colorize, 1, bg, false );
+    QPixmap miniIcon = effect.apply( kapp->miniIcon(), KIconEffect::Colorize, 1, bg, false );
+    KWin::setIcons( winId(), icon, miniIcon );
+
+    // set the color for the selection used to highlight the find stuff
+    QColor sel = palette().color( QPalette::Active, QColorGroup::Base ).dark();
+    if ( sel == Qt::black )
+        sel = palette().color( QPalette::Active, QColorGroup::Base ).light();
+
+    m_editor->setSelectionAttributes( 1, sel, true );
+
+    // update the color of the fold
+    createFold();
+
+    // update the color of the title
+    updateFocus();
+    emit sigColorChanged();
+}
+
 void KNote::find( const QString& pattern, long options )
 {
     delete m_find;
@@ -622,9 +700,11 @@ void KNote::slotUpdateReadOnly()
     actionCollection()->action( "insert_date" )->setEnabled( !readOnly );
     actionCollection()->action( "delete_note" )->setEnabled( !readOnly );
 
-    // TODO: replace the menu
-    actionCollection()->action( "edit_clear" )->setEnabled( !readOnly );
+    actionCollection()->action( "edit_undo" )->setEnabled( !readOnly && m_editor->isUndoAvailable() );
+    actionCollection()->action( "edit_redo" )->setEnabled( !readOnly && m_editor->isRedoAvailable() );
+    actionCollection()->action( "edit_cut" )->setEnabled( !readOnly && m_editor->hasSelectedText() );
     actionCollection()->action( "edit_paste" )->setEnabled( !readOnly );
+    actionCollection()->action( "edit_clear" )->setEnabled( !readOnly );
 
     updateFocus();
 }
@@ -920,61 +1000,6 @@ void KNote::toDesktop( int desktop )
         KWin::setOnDesktop( winId(), desktop );
 }
 
-void KNote::setColor( const QColor &fg, const QColor &bg )
-{
-    QPalette newpalette = palette();
-    newpalette.setColor( QColorGroup::Background, bg );
-    newpalette.setColor( QColorGroup::Foreground, fg );
-    newpalette.setColor( QColorGroup::Base,       bg ); // text background
-    newpalette.setColor( QColorGroup::Text,       fg ); // text color
-    newpalette.setColor( QColorGroup::Button,     bg );
-    newpalette.setColor( QColorGroup::ButtonText, fg );
-    
-//    newpalette.setColor( QColorGroup::Highlight,  bg );
-//    newpalette.setColor( QColorGroup::HighlightedText, fg );
-
-    // the shadow
-    newpalette.setColor( QColorGroup::Midlight, bg.light(150) );
-    newpalette.setColor( QColorGroup::Shadow, bg.dark(116) );
-    newpalette.setColor( QColorGroup::Light, bg.light(180) );
-    if ( s_ppOffset )
-        newpalette.setColor( QColorGroup::Dark, bg.dark(200) );
-    else
-        newpalette.setColor( QColorGroup::Dark, bg.dark(108) );
-    setPalette( newpalette );
-
-    // set the text color
-    m_editor->setTextColor( fg );
-
-    // set the background color or gradient
-    updateBackground();
-
-    // set darker value for the hide button...
-    QPalette darker = palette();
-    darker.setColor( QColorGroup::Button, bg.dark(116) );
-    m_button->setPalette( darker );
-
-    // update the icon color
-    KIconEffect effect;
-    QPixmap icon = effect.apply( kapp->icon(), KIconEffect::Colorize, 1, bg, false );
-    QPixmap miniIcon = effect.apply( kapp->miniIcon(), KIconEffect::Colorize, 1, bg, false );
-    KWin::setIcons( winId(), icon, miniIcon );
-
-    // set the color for the selection used to highlight the find stuff
-    QColor sel = palette().color( QPalette::Active, QColorGroup::Base ).dark();
-    if ( sel == Qt::black )
-        sel = palette().color( QPalette::Active, QColorGroup::Base ).light();
-
-    m_editor->setSelectionAttributes( 1, sel, true );
-
-    // update the color of the fold
-    createFold();
-
-    // update the color of the title
-    updateFocus();
-    emit sigColorChanged();
-}
-
 void KNote::createFold()
 {
     QPixmap fold( 15, 15 );
@@ -1226,18 +1251,18 @@ void KNote::closeEvent( QCloseEvent * )
 
 void KNote::dragEnterEvent( QDragEnterEvent *e )
 {
-    e->accept( KColorDrag::canDecode( e ) );
+    if ( !m_config->readOnly() )
+        e->accept( KColorDrag::canDecode( e ) );
 }
 
 void KNote::dropEvent( QDropEvent *e )
 {
+    if ( m_config->readOnly() )
+        return;
+
     QColor bg;
     if ( KColorDrag::decode( e, bg ) )
-    {
         setColor( paletteForegroundColor(), bg );
-        m_journal->setCustomProperty( "KNotes", "BgColor", bg.name() );
-        m_config->setBgColor( bg );
-    }
 }
 
 bool KNote::focusNextPrevChild( bool )
@@ -1291,6 +1316,15 @@ bool KNote::eventFilter( QObject *o, QEvent *ev )
             return true;
         }
 
+#if KDE_IS_VERSION( 3, 5, 1 )
+        if ( ev->type() == QEvent::MouseButtonRelease )
+        {
+            NETRootInfo wm_root( qt_xdisplay(), NET::WMMoveResize );
+            wm_root.moveResizeRequest( winId(), e->globalX(), e->globalY(), NET::MoveResizeCancel );
+            return false;
+        }
+#endif
+
         if ( m_menu && ( ev->type() == QEvent::MouseButtonPress )
             && ( e->button() == RightButton ) )
         {
