@@ -87,6 +87,7 @@ KOAgendaItem::KOAgendaItem( Incidence *incidence, const QDate &qd, QWidget *pare
 
 void KOAgendaItem::updateIcons()
 {
+  if ( !mIncidence ) return;
   mIconReadonly = mIncidence->isReadOnly();
   mIconRecur = mIncidence->doesRecur();
   mIconAlarm = mIncidence->isAlarmEnabled();
@@ -549,6 +550,7 @@ void KOAgendaItem::dragEnterEvent( QDragEnterEvent *e )
 void KOAgendaItem::addAttendee( const QString &newAttendee )
 {
   kdDebug(5850) << " Email: " << newAttendee << endl;
+  // TODO: Use proper email parsing instead of simply grepping for "<" and "@"
   int pos = newAttendee.find("<");
   QString name = newAttendee.left(pos);
   QString email = newAttendee.mid(pos);
@@ -660,10 +662,23 @@ static void conditionalPaint( QPainter *p, bool cond, int &x, int ft,
   x += pxmp.width() + ft;
 }
 
+void KOAgendaItem::paintEventIcon( QPainter *p, int &x, int ft )
+{
+  if ( !mIncidence ) return;
+  static const QPixmap eventPxmp =
+    KOGlobals::self()->smallIcon( "appointment" );
+  if ( mIncidence->type() != "Event" )
+    return;
+  conditionalPaint( p, true, x, ft, eventPxmp );
+}
+
 void KOAgendaItem::paintTodoIcon( QPainter *p, int &x, int ft )
 {
-  static const QPixmap todoPxmp = KOGlobals::self()->smallIcon("todo");
-  static const QPixmap completedPxmp = KOGlobals::self()->smallIcon("checkedbox");
+  if ( !mIncidence ) return;
+  static const QPixmap todoPxmp =
+    KOGlobals::self()->smallIcon( "todo" );
+  static const QPixmap completedPxmp =
+    KOGlobals::self()->smallIcon( "checkedbox" );
   if ( mIncidence->type() != "Todo" )
     return;
   bool b = ( static_cast<Todo *>( mIncidence ) )->isCompleted();
@@ -673,6 +688,7 @@ void KOAgendaItem::paintTodoIcon( QPainter *p, int &x, int ft )
 
 void KOAgendaItem::paintIcons( QPainter *p, int &x, int ft )
 {
+  paintEventIcon( p, x, ft );
   paintTodoIcon( p, x, ft );
   conditionalPaint( p, mIconAlarm,          x, ft, *alarmPxmp );
   conditionalPaint( p, mIconRecur,          x, ft, *recurPxmp );
@@ -685,6 +701,14 @@ void KOAgendaItem::paintIcons( QPainter *p, int &x, int ft )
 
 void KOAgendaItem::paintEvent( QPaintEvent * )
 {
+  //HACK
+  // to reproduce a crash:
+  // 1. start Kontact with the Calendar as the initial module
+  // 2. immediately select the summary (which must include appt and to-do)
+  // causes a crash for me every time in this method unless we make
+  // the following check
+  if ( !mIncidence )return;
+
   QPainter p( this );
   const int ft = 2; // frame thickness for layout, see paintFrame()
   const int margin = 1 + ft; // frame + space between frame and content
