@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef MK_NNTP_PROTO_H
@@ -39,7 +39,14 @@ public:
 	virtual bool canReadMail() const { return true; }
 	virtual bool fullMessage() const { return true; }
 
-	virtual unsigned short defaultPort() const { return 119; }
+	//virtual int fields() const { return server | port | username | password | mailbox; }
+	//virtual int urlFields() const { return no_fields; }
+	virtual unsigned short defaultPort( bool ) const { return 119; }
+
+	virtual void configFillGroupBoxes( QStringList* ) const;
+        virtual void configFields( QPtrVector< QWidget >* vector, const QObject*, QPtrList< AccountInput >* ) const;
+        virtual void readEntries( QMap< QString, QString >*, QMap< QString, QString >* ) const;
+        virtual void writeEntries( QMap< QString, QString >* ) const;
 };
 
 #endif
