@@ -1,4 +1,4 @@
-/* vcal-conduit.cc                      KPilot
+/* KPilot
 **
 ** Copyright (C) 2002-2003 Reinhold Kainhofer
 ** Copyright (C) 2001 by Dan Pilone
@@ -19,8 +19,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -28,10 +28,12 @@
 */
 
 #include "options.h"
+#include <qtextcodec.h>
 #include <libkcal/calendar.h>
 #include <libkcal/recurrence.h>
 #define Recurrence_t KCal::Recurrence
 #include <pilotDateEntry.h>
+#include <pilotDatabase.h>
 #include "vcal-conduit.moc"
 #include "vcal-factory.h"
 
@@ -39,7 +41,7 @@ extern "C"
 {
 
 long version_conduit_vcal = KPILOT_PLUGIN_API;
-const char *id_conduit_vcal = "$Id: vcal-conduit.cc,v 1.103.2.1 2004/09/29 09:20:54 adridg Exp $";
+const char *id_conduit_vcal = "$Id: vcal-conduit.cc 450417 2005-08-18 06:07:11Z vanrijn $";
 
 }
 
@@ -129,19 +131,23 @@ KCal::Incidence *VCalConduitPrivate::getNextModifiedIncidence()
 	KCal::Event*e=0L;
 	if (!reading)
 	{
+		// Start from the top
 		reading=true;
 		fAllEventsIterator = fAllEvents.begin();
-		if ( fAllEventsIterator != fAllEvents.end() ) e = *fAllEventsIterator;
 	}
 	else
 	{
+		// Move on from current position
 		++fAllEventsIterator;
 	}
+
+	// Fetch (new) current if possible.
+	if ( fAllEventsIterator != fAllEvents.end() ) e = *fAllEventsIterator;
+	// Then walk the list until we find an unsynced entry
 	while ( fAllEventsIterator != fAllEvents.end() &&
-		e && e->syncStatus()==KCal::Incidence::SYNCNONE && e->pilotId() > 0)
+		e && e->syncStatus()!=KCal::Incidence::SYNCMOD && e->pilotId() > 0)
 	{
-		++fAllEventsIterator;
-		e=*fAllEventsIterator;
+		e = (++fAllEventsIterator != fAllEvents.end()) ? *fAllEventsIterator : 0L;
 	}
 	return (fAllEventsIterator == fAllEvents.end()) ? 0L : 	*fAllEventsIterator;
 }
@@ -174,6 +180,32 @@ VCalConduitPrivateBase* VCalConduit::newVCalPrivate(KCal::Calendar *fCalendar) {
 	return new VCalConduitPrivate(fCalendar);
 }
 
+void VCalConduit::_getAppInfo()
+{
+	FUNCTIONSETUP;
+	// get the address application header information
+	unsigned char *buffer =
+		new unsigned char[PilotRecord::APP_BUFFER_SIZE];
+	int appLen = fDatabase->readAppBlock(buffer,PilotRecord::APP_BUFFER_SIZE);
+
+	unpack_AppointmentAppInfo(&fAppointmentAppInfo, buffer, appLen);
+	delete[]buffer;
+	buffer = NULL;
+
+#ifdef DEBUG
+	DEBUGCONDUIT << fname << " lastUniqueId"
+		<< fAppointmentAppInfo.category.lastUniqueID << endl;
+#endif
+	for (int i = 0; i < 16; i++)
+	{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << " cat " << i << " =" <<
+			fAppointmentAppInfo.category.name[i] << endl;
+#endif
+	}
+
+}
+
 const QString VCalConduit::getTitle(PilotAppCategory*de)
 {
 	PilotDateEntry*d=dynamic_cast<PilotDateEntry*>(de);
@@ -207,7 +239,7 @@ PilotRecord*VCalConduit::recordFromIncidence(PilotDateEntry*de, const KCal::Even
 	}
 
 	// set secrecy, start/end times, alarms, recurrence, exceptions, summary and description:
-	if (e->secrecy()!=KCal::Event::SecrecyPublic) de->makeSecret();
+	if (e->secrecy()!=KCal::Event::SecrecyPublic) de->setSecret( true );
 
 	setStartEndTimes(de, e);
 	setAlarms(de, e);
@@ -215,6 +247,7 @@ PilotRecord*VCalConduit::recordFromIncidence(PilotDateEntry*de, const KCal::Even
 	setExceptions(de, e);
 	de->setDescription(e->summary());
 	de->setNote(e->description());
+	setCategory(de, e);
 DEBUGCONDUIT<<"-------- "<<e->summary()<<endl;
 	return de->pack();
 }
@@ -243,7 +276,10 @@ KCal::Event *VCalConduit::incidenceFromRecord(KCal::Event *e, const PilotDateEnt
 		KCal::Event::SecrecyPrivate :
 		KCal::Event::SecrecyPublic);
 
-	e->setPilotId(de->getID());
+	e->setPilotId(de->id());
+#ifdef DEBUG
+		DEBUGCONDUIT<<fname<<": set KCal item to pilotId: [" << e->pilotId() << "]..."<<endl;
+#endif
 	e->setSyncStatus(KCal::Incidence::SYNCNONE);
 
 	setStartEndTimes(e,de);
@@ -257,6 +293,9 @@ KCal::Event *VCalConduit::incidenceFromRecord(KCal::Event *e, const PilotDateEnt
 #endif
 	e->setDescription(de->getNote());
 
+	// used by e.g. Agendus and Datebk
+	setCategory(e, de);
+
 	return e;
 }
 
@@ -286,7 +325,7 @@ void VCalConduit::setStartEndTimes(PilotDateEntry*de, const KCal::Event *e)
 	FUNCTIONSETUP;
 	struct tm ttm=writeTm(e->dtStart());
 	de->setEventStart(ttm);
-	de->setEvent(e->doesFloat());
+	de->setFloats( e->doesFloat() );
 
 	if (e->hasEndDate() && e->dtEnd().isValid())
 	{
@@ -307,7 +346,7 @@ void VCalConduit::setAlarms(KCal::Event *e, const PilotDateEntry *de)
 	if (!e) return;
 	// Delete all the alarms now and add them one by one later on.
 	e->clearAlarms();
-	if (!de->getAlarm()) return;
+	if (!de->isAlarmEnabled()) return;
 
 //	QDateTime alarmDT = readTm(de->getEventStart());
 	int advanceUnits = de->getAdvanceUnits();
@@ -357,7 +396,7 @@ void VCalConduit::setAlarms(PilotDateEntry*de, const KCal::Event *e)
 
 	if ( !e->isAlarmEnabled() )
 	{
-		de->setAlarm(0);
+		de->setAlarmEnabled( false );
 		return;
 	}
 
@@ -374,7 +413,7 @@ void VCalConduit::setAlarms(PilotDateEntry*de, const KCal::Event *e)
 #ifdef DEBUG
 		DEBUGCONDUIT << fname << ": no enabled alarm found (should exist!!!)"<<endl;
 #endif
-		de->setAlarm(0);
+		de->setAlarmEnabled( false );
 		return;
 	}
 
@@ -401,7 +440,7 @@ void VCalConduit::setAlarms(PilotDateEntry*de, const KCal::Event *e)
 		de->setAdvanceUnits(advMinutes);
 	}
 	de->setAdvance((aoffs>0)?offs:-offs);
-	de->setAlarm(1);
+	de->setAlarmEnabled( true );
 }
 
 
@@ -427,11 +466,9 @@ void VCalConduit::setRecurrence(KCal::Event *event,const PilotDateEntry *dateEnt
 		endDate = readTm(dateEntry->getRepeatEnd()).date();
 #ifdef DEBUG
 		DEBUGCONDUIT << fname << "-- end " << endDate.toString() << endl;
-#endif
 	}
 	else
 	{
-#ifdef DEBUG
 		DEBUGCONDUIT << fname << "-- noend" << endl;
 #endif
 	}
@@ -441,8 +478,7 @@ void VCalConduit::setRecurrence(KCal::Event *event,const PilotDateEntry *dateEnt
 	switch(dateEntry->getRepeatType())
 	{
 	case repeatDaily:
-		if (repeatsForever) recur->setDaily(freq,-1);
-		else recur->setDaily(freq,endDate);
+		recur->setDaily(freq);
 		break;
 	case repeatWeekly:
 		{
@@ -465,9 +501,7 @@ void VCalConduit::setRecurrence(KCal::Event *event,const PilotDateEntry *dateEnt
 		{
 			if (days[i]) dayArray.setBit(i-1);
 		}
-
-		if (repeatsForever) recur->setWeekly(freq,dayArray,-1);
-		else recur->setWeekly(freq,dayArray,endDate);
+		recur->setWeekly( freq, dayArray );
 		}
 		break;
 	case repeatMonthlyByDay: {
@@ -476,14 +510,7 @@ void VCalConduit::setRecurrence(KCal::Event *event,const PilotDateEntry *dateEnt
 		// Palm->PC: w=pos/7
 		// week: if w=4 -> week=-1, else week=w+1;
 		// day: day=(pos-1)%7 (rotate by one day!)
-		if (repeatsForever)
-		{
-			recur->setMonthly(Recurrence_t::rMonthlyPos,freq,-1);
-		}
-		else
-		{
-			recur->setMonthly(Recurrence_t::rMonthlyPos,freq,endDate);
-		}
+		recur->setMonthly( freq );
 
 		int day=dateEntry->getRepeatDay();
 		int week=day/7;
@@ -493,27 +520,13 @@ void VCalConduit::setRecurrence(KCal::Event *event,const PilotDateEntry *dateEnt
 		recur->addMonthlyPos(week, dayArray);
 		break;}
 	case repeatMonthlyByDate:
-		if (repeatsForever)
-		{
-			recur->setMonthly(Recurrence_t::rMonthlyDay,freq,-1);
-		}
-		else
-		{
-			recur->setMonthly(Recurrence_t::rMonthlyDay,freq,endDate);
-		}
-		recur->addMonthlyDay( dateEntry->getEventStart().tm_mday );
+		recur->setMonthly( freq );
+		recur->addMonthlyDate( dateEntry->getEventStart().tm_mday );
 		break;
 	case repeatYearly:
-		if (repeatsForever)
-		{
-			recur->setYearly(Recurrence_t::rYearlyMonth,freq,-1);
-		}
-		else
-		{
-			recur->setYearly(Recurrence_t::rYearlyMonth,freq,endDate);
-		}
+		recur->setYearly( freq );
 		evt=readTm(dateEntry->getEventStart()).date();
-		recur->addYearlyNum( evt.month() );
+		recur->addYearlyMonth( evt.month() );
 //		dayArray.setBit((evt.day()-1) % 7);
 //		recur->addYearlyMonthPos( ( (evt.day()-1) / 7) + 1, dayArray );
 		break;
@@ -527,6 +540,10 @@ void VCalConduit::setRecurrence(KCal::Event *event,const PilotDateEntry *dateEnt
 #endif
 		break;
 	}
+	if (!repeatsForever)
+	{
+		recur->setEndDate(endDate);
+	}
 }
 
 
@@ -535,7 +552,7 @@ void VCalConduit::setRecurrence(PilotDateEntry*dateEntry, const KCal::Event *eve
 	FUNCTIONSETUP;
 	bool isMultiDay=false;
 
-	//  first we have 'fake type of recurrence' when a multi-day event is passed to the pilot, it is converted to an event
+	// first we have 'fake type of recurrence' when a multi-day event is passed to the pilot, it is converted to an event
 	// which recurs daily a number of times. if the event itself recurs, this will be overridden, and
 	// only the first day will be included in the event!!!!
 	QDateTime startDt(readTm(dateEntry->getEventStart())), endDt(readTm(dateEntry->getEventEnd()));
@@ -553,8 +570,8 @@ void VCalConduit::setRecurrence(PilotDateEntry*dateEntry, const KCal::Event *eve
 
 	KCal::Recurrence*r=event->recurrence();
 	if (!r) return;
-	ushort recType=r->doesRecur();
-	if (recType==KCal::Recurrence::rNone)
+	ushort recType=r->recurrenceType();
+	if ( recType==KCal::Recurrence::rNone )
 	{
 		if (!isMultiDay) dateEntry->setRepeatType(repeatNone);
 		return;
@@ -564,7 +581,7 @@ void VCalConduit::setRecurrence(PilotDateEntry*dateEntry, const KCal::Event *eve
 	int freq=r->frequency();
 	QDate endDate=r->endDate();
 
-	if (!endDate.isValid())
+	if ( r->duration() < 0 || !endDate.isValid() )
 	{
 		dateEntry->setRepeatForever();
 	}
@@ -604,17 +621,12 @@ void VCalConduit::setRecurrence(PilotDateEntry*dateEntry, const KCal::Event *eve
 		if (r->monthPositions().count()>0)
 		{
 			// Only take the first monthly position, as the palm allows only one
-			QPtrList<KCal::Recurrence::rMonthPos> mps=r->monthPositions();
-			const KCal::Recurrence::rMonthPos*mp=mps.first();
-			int pos=0;
-			dayArray=mp->rDays;
-			// this is quite clumsy, but I haven't found a better way...
-			for (int j=0; j<7; j++)
-				if (dayArray[j]) pos=j;
-			int week=mp->rPos;
-			if (mp->negative) week*=-1;
-			int day=(pos+1) % 7; // rotate because of different offset
+			QValueList<KCal::RecurrenceRule::WDayPos> mps=r->monthPositions();
+			KCal::RecurrenceRule::WDayPos mp=mps.first();
+			int week = mp.pos();
+			int day = (mp.day()+1) % 7; // rotate because of different offset
 			// turn to 0-based and include starting from end of month
+			// TODO: We don't handle counting from the end of the month yet!
 			if (week==-1) week=4; else week--;
 			dateEntry->setRepeatDay(static_cast<DayOfMonthType>(7*week + day));
 		}
@@ -669,7 +681,7 @@ void VCalConduit::setExceptions(KCal::Event *vevent,const PilotDateEntry *dateEn
 #endif
 		return;
 	}
-	vevent->setExDates(dl);
+	vevent->recurrence()->setExDates(dl);
 }
 
 void VCalConduit::setExceptions(PilotDateEntry *dateEntry, const KCal::Event *vevent )
@@ -688,7 +700,8 @@ void VCalConduit::setExceptions(PilotDateEntry *dateEntry, const KCal::Event *ve
 	if (ex_List)
 		KPILOT_DELETE(ex_List);*/
 
-	size_t excount=vevent->exDates().size();
+	KCal::DateList exDates = vevent->recurrence()->exDates();
+	size_t excount = exDates.size();
 	if (excount<1)
 	{
 		dateEntry->setExceptionCount(0);
@@ -708,7 +721,6 @@ void VCalConduit::setExceptions(PilotDateEntry *dateEntry, const KCal::Event *ve
 
 	size_t n=0;
 
-	KCal::DateList exDates = vevent->exDates();
 	KCal::DateList::ConstIterator dit;
 	for (dit = exDates.begin(); dit != exDates.end(); ++dit ) {
 		struct tm ttm=writeTm(*dit);
@@ -718,13 +730,71 @@ void VCalConduit::setExceptions(PilotDateEntry *dateEntry, const KCal::Event *ve
 	dateEntry->setExceptions(ex_List);
 }
 
-PilotAppCategory*VCalConduit::newPilotEntry(PilotRecord*r) 
+
+void VCalConduit::setCategory(PilotDateEntry *de, const KCal::Event *e)
 {
-  if (r) return new PilotDateEntry(r);
-	else return new PilotDateEntry();
+	if (!de || !e) return;
+	de->setCategory(_getCat(e->categories(), de->getCategoryLabel()));
 }
 
-KCal::Incidence*VCalConduit::newIncidence() 
+/**
+ * _getCat returns the id of the category from the given categories list. If none of the categories exist
+ * on the palm, the "Nicht abgelegt" (don't know the english name) is used.
+ */
+
+QString VCalConduit::_getCat(const QStringList cats, const QString curr) const
+{
+	int j;
+	if (cats.size()<1) return QString::null;
+	if (cats.contains(curr)) return curr;
+	for ( QStringList::ConstIterator it = cats.begin(); it != cats.end(); ++it ) {
+		for (j=1; j<PILOT_CATEGORY_MAX; j++)
+		{
+			QString catName = PilotAppCategory::codec()->
+        toUnicode(fAppointmentAppInfo.category.name[j]);
+			if (!(*it).isEmpty() && !(*it).compare( catName ) )
+			{
+				return catName;
+			}
+		}
+	}
+	// If we have a free label, return the first possible cat
+	QString lastName(QString::fromLatin1(fAppointmentAppInfo.category.name[PILOT_CATEGORY_MAX-1]));
+	if (lastName.isEmpty()) return cats.first();
+	return QString::null;
+}
+
+void VCalConduit::setCategory(KCal::Event *e, const PilotDateEntry *de)
+{
+	if (!e || !de) return;
+	QStringList cats=e->categories();
+	int cat=de->category();
+	if (0<cat && cat<PILOT_CATEGORY_MAX)
+	{
+		QString newcat=PilotAppCategory::codec()->toUnicode(fAppointmentAppInfo.category.name[cat]);
+		if (!cats.contains(newcat))
+		{
+			// if this event only has one category associated with it, then we can
+			// safely assume that what we should be doing here is changing it to match
+			// the palm.  if there's already more than one category in the event, however, we
+			// won't cause data loss--we'll just append what the palm has to the 
+			// event's categories
+			if (cats.count() <=1) cats.clear();
+			
+			cats.append( newcat );
+			e->setCategories(cats);
+		}
+	}
+}
+
+
+PilotAppCategory*VCalConduit::newPilotEntry(PilotRecord*r)
+{
+	if (r) return new PilotDateEntry(fAppointmentAppInfo,r);
+	else return new PilotDateEntry(fAppointmentAppInfo);
+}
+
+KCal::Incidence*VCalConduit::newIncidence()
 {
   return new KCal::Event;
 }
