@@ -20,8 +20,8 @@
 **
 ** You should have received a copy of the GNU Lesser General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -45,7 +45,7 @@
 #include "pilotSerialDatabase.h"
 
 static const char *pilotSerialDatabase_id =
-	"$Id: pilotSerialDatabase.cc,v 1.37 2005/02/09 15:02:02 adridg Exp $";
+	"$Id: pilotSerialDatabase.cc 437980 2005-07-23 19:53:57Z kainhofe $";
 
 PilotSerialDatabase::PilotSerialDatabase(int linksocket,
 	const QString &dbName) :
@@ -134,7 +134,13 @@ QValueList<recordid_t> PilotSerialDatabase::idList()
 
 	recordid_t *idarr=new recordid_t[idlen];
 	int idlenread;
-	dlp_ReadRecordIDList (fDBSocket, getDBHandle(), 0, 0, idlen, idarr, &idlenread);
+	int r = dlp_ReadRecordIDList (fDBSocket, getDBHandle(), 0, 0, idlen, idarr, &idlenread);
+
+	if ( (r<0) || (idlenread<1) )
+	{
+		kdWarning() << k_funcinfo << ": Failed to read ID list from database." << endl;
+		return idlist;
+	}
 
 	// now create the QValue list from the idarr:
 	for (idlen=0; idlen<idlenread; idlen++)
@@ -298,9 +304,9 @@ recordid_t PilotSerialDatabase::writeRecord(PilotRecord * newRecord)
 	}
 	success =
 		dlp_WriteRecord(fDBSocket, getDBHandle(),
-		newRecord->getAttrib(), newRecord->id(),
-		newRecord->category(), newRecord->getData(),
-		newRecord->getLen(), &newid);
+		newRecord->attributes(), newRecord->id(),
+		newRecord->category(), newRecord->data(),
+		newRecord->size(), &newid);
 	if ( (newRecord->id() != newid) && (newid!=0) )
 		newRecord->setID(newid);
 	return newid;
