@@ -2,6 +2,8 @@
     This file is part of libkdepim.
 
     Copyright (c) 2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
+    Copyright (C) 2005 Allen Winter <winter@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -15,8 +17,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <qlayout.h>
@@ -30,6 +32,7 @@
 #include <qcheckbox.h>
 #include <qradiobutton.h>
 #include <qpushbutton.h>
+#include <qdatetimeedit.h>
 #include <qwhatsthis.h>
 
 #include <kcolorbutton.h>
@@ -38,6 +41,9 @@
 #include <kfontdialog.h>
 #include <kmessagebox.h>
 #include <kconfigskeleton.h>
+#include <kurlrequester.h>
+#include "ktimeedit.h"
+#include "kdateedit.h"
 
 #include "kprefsdialog.h"
 #include "kprefsdialog.moc"
@@ -57,7 +63,7 @@ KPrefsWid *create( KConfigSkeletonItem *item, QWidget *parent )
   if ( stringItem ) {
     return new KPrefsWidString( stringItem, parent );
   }
-  
+
   KConfigSkeleton::ItemEnum *enumItem =
       dynamic_cast<KConfigSkeleton::ItemEnum *>( item );
   if ( enumItem ) {
@@ -74,13 +80,13 @@ KPrefsWid *create( KConfigSkeletonItem *item, QWidget *parent )
       return radios;
     }
   }
-  
+
   KConfigSkeleton::ItemInt *intItem =
       dynamic_cast<KConfigSkeleton::ItemInt *>( item );
   if ( intItem ) {
     return new KPrefsWidInt( intItem, parent );
   }
-  
+
   return 0;
 }
 
@@ -130,8 +136,14 @@ KPrefsWidInt::KPrefsWidInt( KConfigSkeleton::ItemInt *item,
                             QWidget *parent )
   : mItem( item )
 {
-  mLabel = new QLabel( mItem->label(), parent );
+  mLabel = new QLabel( mItem->label()+':', parent );
   mSpin = new QSpinBox( parent );
+  if ( !item->minValue().isNull() ) {
+    mSpin->setMinValue( item->minValue().toInt() );
+  }
+  if ( !item->maxValue().isNull() ) {
+    mSpin->setMaxValue( item->maxValue().toInt() );
+  }
   connect( mSpin, SIGNAL( valueChanged( int ) ), SIGNAL( changed() ) );
   mLabel->setBuddy( mSpin );
   QString whatsThis = mItem->whatsThis();
@@ -176,7 +188,8 @@ KPrefsWidColor::KPrefsWidColor( KConfigSkeleton::ItemColor *item,
 {
   mButton = new KColorButton( parent );
   connect( mButton, SIGNAL( changed( const QColor & ) ), SIGNAL( changed() ) );
-  mLabel = new QLabel( mButton, mItem->label(), parent );
+  mLabel = new QLabel( mButton, mItem->label()+':', parent );
+  mLabel->setBuddy( mButton );
   QString whatsThis = mItem->whatsThis();
   if (!whatsThis.isNull()) {
     QWhatsThis::add(mButton, whatsThis);
@@ -213,7 +226,7 @@ KPrefsWidFont::KPrefsWidFont( KConfigSkeleton::ItemFont *item,
                               QWidget *parent, const QString &sampleText )
   : mItem( item )
 {
-  mLabel = new QLabel( mItem->label(), parent );
+  mLabel = new QLabel( mItem->label()+':', parent );
 
   mPreview = new QLabel( sampleText, parent );
   mPreview->setFrameStyle( QFrame::Panel | QFrame::Sunken );
@@ -267,28 +280,32 @@ void KPrefsWidFont::selectFont()
 }
 
 
-KPrefsWidTime::KPrefsWidTime( KConfigSkeleton::ItemInt *item,
+KPrefsWidTime::KPrefsWidTime( KConfigSkeleton::ItemDateTime *item,
                               QWidget *parent )
   : mItem( item )
 {
-  mLabel = new QLabel( mItem->label(), parent );
-  mSpin = new QSpinBox( 0, 23, 1, parent );
-  connect( mSpin, SIGNAL( valueChanged( int ) ), SIGNAL( changed() ) );
-  mSpin->setSuffix( ":00" );
+  mLabel = new QLabel( mItem->label()+':', parent );
+  mTimeEdit = new KTimeEdit( parent );
+  mLabel->setBuddy( mTimeEdit );
+  connect( mTimeEdit, SIGNAL( timeChanged( QTime ) ), SIGNAL( changed() ) );
   QString whatsThis = mItem->whatsThis();
   if ( !whatsThis.isNull() ) {
-    QWhatsThis::add( mSpin, whatsThis );
+    QWhatsThis::add( mTimeEdit, whatsThis );
   }
 }
 
 void KPrefsWidTime::readConfig()
 {
-  mSpin->setValue( mItem->value() );
+  mTimeEdit->setTime( mItem->value().time() );
 }
 
 void KPrefsWidTime::writeConfig()
 {
-  mItem->setValue( mSpin->value() );
+  // Don't overwrite the date value of the QDateTime, so we can use a
+  // KPrefsWidTime and a KPrefsWidDate on the same config entry!
+  QDateTime dt( mItem->value() );
+  dt.setTime( mTimeEdit->getTime() );
+  mItem->setValue( dt );
 }
 
 QLabel *KPrefsWidTime::label()
@@ -296,9 +313,87 @@ QLabel *KPrefsWidTime::label()
   return mLabel;
 }
 
-QSpinBox *KPrefsWidTime::spinBox()
+KTimeEdit *KPrefsWidTime::timeEdit()
 {
-  return mSpin;
+  return mTimeEdit;
+}
+
+
+KPrefsWidDuration::KPrefsWidDuration( KConfigSkeleton::ItemDateTime *item,
+                                      QWidget *parent )
+  : mItem( item )
+{
+  mLabel = new QLabel( mItem->label()+':', parent );
+  mTimeEdit = new QTimeEdit( parent );
+  mLabel->setBuddy( mTimeEdit );
+  mTimeEdit->setAutoAdvance( true );
+  mTimeEdit->setDisplay( QTimeEdit::Hours | QTimeEdit::Minutes );
+  mTimeEdit->setRange( QTime( 0, 1 ), QTime( 24, 0 ) ); // [1min, 24hr]
+  connect( mTimeEdit,
+           SIGNAL( valueChanged( const QTime & ) ), SIGNAL( changed() ) );
+  QString whatsThis = mItem->whatsThis();
+  if ( !whatsThis.isNull() ) {
+    QWhatsThis::add( mTimeEdit, whatsThis );
+  }
+}
+
+void KPrefsWidDuration::readConfig()
+{
+  mTimeEdit->setTime( mItem->value().time() );
+}
+
+void KPrefsWidDuration::writeConfig()
+{
+  QDateTime dt( mItem->value() );
+  dt.setTime( mTimeEdit->time() );
+  mItem->setValue( dt );
+}
+
+QLabel *KPrefsWidDuration::label()
+{
+  return mLabel;
+}
+
+QTimeEdit *KPrefsWidDuration::timeEdit()
+{
+  return mTimeEdit;
+}
+
+
+KPrefsWidDate::KPrefsWidDate( KConfigSkeleton::ItemDateTime *item,
+                              QWidget *parent )
+  : mItem( item )
+{
+  mLabel = new QLabel( mItem->label()+':', parent );
+  mDateEdit = new KDateEdit( parent );
+  mLabel->setBuddy( mDateEdit );
+  connect( mDateEdit, SIGNAL( dateChanged( const QDate& ) ), SIGNAL( changed() ) );
+  QString whatsThis = mItem->whatsThis();
+  if ( !whatsThis.isNull() ) {
+    QWhatsThis::add( mDateEdit, whatsThis );
+  }
+}
+
+void KPrefsWidDate::readConfig()
+{
+  mDateEdit->setDate( mItem->value().date() );
+}
+
+void KPrefsWidDate::writeConfig()
+{
+  QDateTime dt( mItem->value() );
+  dt.setDate( mDateEdit->date() );
+  mItem->setValue( dt );
+}
+
+QLabel *KPrefsWidDate::label()
+{
+  return mLabel;
+}
+
+KDateEdit *KPrefsWidDate::dateEdit()
+{
+  return mDateEdit;
 }
 
 
@@ -350,8 +445,9 @@ KPrefsWidString::KPrefsWidString( KConfigSkeleton::ItemString *item,
                                   QLineEdit::EchoMode echomode )
   : mItem( item )
 {
-  mLabel = new QLabel( mItem->label(), parent );
+  mLabel = new QLabel( mItem->label()+':', parent );
   mEdit = new QLineEdit( parent );
+  mLabel->setBuddy( mEdit );
   connect( mEdit, SIGNAL( textChanged( const QString & ) ),
            SIGNAL( changed() ) );
   mEdit->setEchoMode( echomode );
@@ -394,6 +490,56 @@ QValueList<QWidget *> KPrefsWidString::widgets() const
 }
 
 
+KPrefsWidPath::KPrefsWidPath( KConfigSkeleton::ItemPath *item, QWidget *parent,
+                              const QString &filter, uint mode )
+  : mItem( item )
+{
+  mLabel = new QLabel( mItem->label()+':', parent );
+  mURLRequester = new KURLRequester( parent );
+  mLabel->setBuddy( mURLRequester );
+  mURLRequester->setMode( mode );
+  mURLRequester->setFilter( filter );
+  connect( mURLRequester, SIGNAL( textChanged( const QString & ) ),
+           SIGNAL( changed() ) );
+  QString whatsThis = mItem->whatsThis();
+  if ( !whatsThis.isNull() ) {
+    QWhatsThis::add( mURLRequester, whatsThis );
+  }
+}
+
+KPrefsWidPath::~KPrefsWidPath()
+{
+}
+
+void KPrefsWidPath::readConfig()
+{
+  mURLRequester->setURL( mItem->value() );
+}
+
+void KPrefsWidPath::writeConfig()
+{
+  mItem->setValue( mURLRequester->url() );
+}
+
+QLabel *KPrefsWidPath::label()
+{
+  return mLabel;
+}
+
+KURLRequester *KPrefsWidPath::urlRequester()
+{
+  return mURLRequester;
+}
+
+QValueList<QWidget *> KPrefsWidPath::widgets() const
+{
+  QValueList<QWidget *> widgets;
+  widgets.append( mLabel );
+  widgets.append( mURLRequester );
+  return widgets;
+}
+
+
 KPrefsWidManager::KPrefsWidManager( KConfigSkeleton *prefs )
   : mPrefs( prefs )
 {
@@ -416,7 +562,7 @@ KPrefsWidBool *KPrefsWidManager::addWidBool( KConfigSkeleton::ItemBool *item,
   return w;
 }
 
-KPrefsWidTime *KPrefsWidManager::addWidTime( KConfigSkeleton::ItemInt *item,
+KPrefsWidTime *KPrefsWidManager::addWidTime( KConfigSkeleton::ItemDateTime *item,
                                              QWidget *parent )
 {
   KPrefsWidTime *w = new KPrefsWidTime( item, parent );
@@ -424,6 +570,22 @@ KPrefsWidTime *KPrefsWidManager::addWidTime( KConfigSkeleton::ItemInt *item,
   return w;
 }
 
+KPrefsWidDuration *KPrefsWidManager::addWidDuration( KConfigSkeleton::ItemDateTime *item,
+                                                     QWidget *parent )
+{
+  KPrefsWidDuration *w = new KPrefsWidDuration( item, parent );
+  addWid( w );
+  return w;
+}
+
+KPrefsWidDate *KPrefsWidManager::addWidDate( KConfigSkeleton::ItemDateTime *item,
+                                             QWidget *parent )
+{
+  KPrefsWidDate *w = new KPrefsWidDate( item, parent );
+  addWid( w );
+  return w;
+}
+
 KPrefsWidColor *KPrefsWidManager::addWidColor( KConfigSkeleton::ItemColor *item,
                                                QWidget *parent )
 {
@@ -455,6 +617,14 @@ KPrefsWidString *KPrefsWidManager::addWidString( KConfigSkeleton::ItemString *it
   return w;
 }
 
+KPrefsWidPath *KPrefsWidManager::addWidPath( KConfigSkeleton::ItemPath *item,
+                                             QWidget *parent, const QString &filter, uint mode )
+{
+  KPrefsWidPath *w = new KPrefsWidPath( item, parent, filter, mode );
+  addWid( w );
+  return w;
+}
+
 KPrefsWidString *KPrefsWidManager::addWidPassword( KConfigSkeleton::ItemString *item,
                                                    QWidget *parent )
 {
@@ -472,6 +642,14 @@ KPrefsWidFont *KPrefsWidManager::addWidFont( KConfigSkeleton::ItemFont *item,
   return w;
 }
 
+KPrefsWidInt *KPrefsWidManager::addWidInt( KConfigSkeleton::ItemInt *item,
+                                           QWidget *parent )
+{
+  KPrefsWidInt *w = new KPrefsWidInt( item, parent );
+  addWid( w );
+  return w;
+}
+
 void KPrefsWidManager::setWidDefaults()
 {
   kdDebug() << "KPrefsWidManager::setWidDefaults()" << endl;
@@ -526,11 +704,11 @@ KPrefsDialog::~KPrefsDialog()
 void KPrefsDialog::autoCreate()
 {
   KConfigSkeletonItem::List items = prefs()->items();
-  
+
   QMap<QString,QWidget *> mGroupPages;
   QMap<QString,QGridLayout *> mGroupLayouts;
   QMap<QString,int> mCurrentRows;
-  
+
   KConfigSkeletonItem::List::ConstIterator it;
   for( it = items.begin(); it != items.end(); ++it ) {
     QString group = (*it)->group();
@@ -555,7 +733,7 @@ void KPrefsDialog::autoCreate()
     }
 
     KPrefsWid *wid = KPrefsWidFactory::create( *it, page );
-    
+
     if ( wid ) {
       QValueList<QWidget *> widgets = wid->widgets();
       if ( widgets.count() == 1 ) {
@@ -563,24 +741,24 @@ void KPrefsDialog::autoCreate()
                                     currentRow, currentRow, 0, 1 );
       } else if ( widgets.count() == 2 ) {
         layout->addWidget( widgets[ 0 ], currentRow, 0 );
-        layout->addWidget( widgets[ 1 ], currentRow, 1 );      
+        layout->addWidget( widgets[ 1 ], currentRow, 1 );
       } else {
         kdError() << "More widgets than expected: " << widgets.count() << endl;
       }
-  
+
       if ( (*it)->isImmutable() ) {
         QValueList<QWidget *>::Iterator it2;
         for( it2 = widgets.begin(); it2 != widgets.end(); ++it2 ) {
           (*it2)->setEnabled( false );
         }
       }
-  
+
       addWid( wid );
-  
+
       mCurrentRows.replace( group, ++currentRow );
     }
   }
-  
+
   readConfig();
 }
 
@@ -625,7 +803,7 @@ void KPrefsDialog::slotDefault()
   if (KMessageBox::warningContinueCancel(this,
       i18n("You are about to set all preferences to default values. All "
       "custom modifications will be lost."),i18n("Setting Default Preferences"),
-      i18n("Continue"))
+      i18n("Reset to Defaults"))
     == KMessageBox::Continue) setDefaults();
 }
 
